@@ -6,7 +6,7 @@
 #
 #	Copyright 2001-2002, Balazs Barany balazs@tud.at
 #
-#	Version 0.6.4
+#	Version 0.6.9
 #
 #	This program is free software; you can redistribute it and/or
 #	modify it under the terms of the GNU General Public License as
@@ -43,7 +43,7 @@
 #                       e.g. 192.168.0.1/tcp/21 or firewall/udp/137		
 #
 #
-#	$Id: fwanalog.sh,v 1.75 2004/03/18 16:40:17 bb Exp $
+#	$Id: fwanalog.sh,v 1.81 2005/02/24 16:59:44 bb Exp $
 #
 #	Revisions:
 #
@@ -85,6 +85,8 @@
 #	2003-11-25	Version 0.6.4pre4: Smaller fixes, mainly for PIX
 #	2004-03-18	Version 0.6.4: PIX fixes, released as 0.6.4 on the request of
 #										the Debian maintainer
+#	2005-02-24	Version 0.6.9: PIX fix, added contributed ipfw, sonicwall
+#										parsers
 #
 ###########################################################################
 
@@ -95,7 +97,7 @@
 # Script options
 
 PROGNAME=$(basename $0)
-VERSION="0.6.4"
+VERSION="0.6.9"
 
 ###########################################################################
 #	Variables
@@ -163,6 +165,17 @@ main ()
 	
 	# create the output directory if necessary, ignore errors
 	mkdir -p $outdir
+
+    # Check if the lock file is there. If yes, warn and exit.
+    if [ -e $outdir/fwanalog.lock ]; then
+		echo "fwanalog: found lockfile '$outdir/fwanalog.lock'. " >> /dev/stderr
+		echo "This could mean that another instance is running." >> /dev/stderr
+		echo "If this is not the case, please remove the lock file." >> /dev/stderr
+		exit 1
+    fi
+
+    # Create lock file
+    touch $outdir/fwanalog.lock
 	
 	# Parse the logs into a format Analog understands
 	$logformat
@@ -175,16 +188,16 @@ main ()
 	# there is already an old log - find its last line and use it
 	# to determine the new contents of the grepped/converted file
 		$grep . $outdir/fwanalog.all.log \
-			| tail -1 \
+			| tail -n 1 \
 			| $sed 's/[^a-zA-Z0-9 _-]/./g' \
-			| $sed 's#^\(.*\)$#1,/^\1$/d#' \
+			| $sed 's#^\(.*\)$#0,/^\1$/d#' \
 			> $outdir/match_last_line.sed
 			# match_last_line.sed now contains the last line in regexp form, so
 			# it can be searched in the new file. Most non-alphanumeric chars
 			# have been replaced with . so they don't act as metacharacters.
 
 		$grep . $outdir/fwanalog.all.log \
-			| tail -1 \
+			| tail -n 1 \
 			| $sed 's/[^a-zA-Z0-9 _-]/./g' \
 			> $outdir/match_last_line.pattern
 			# create the regexp for grep
@@ -195,6 +208,7 @@ main ()
 		# Check if there is a common part in the old an the new log
 		if $grep --silent "`cat $outdir/match_last_line.pattern`" $outdir/fwanalog.current.log ; then
 		# there is a common part
+        
 			# Delete the common lines in the current log so only the new ones
 			# stay and write it to the end of the global log
 			$sed -f $outdir/match_last_line.sed $outdir/fwanalog.current.log \
@@ -610,6 +624,9 @@ main ()
 	# Clean up old logfiles
 	rm -f $outdir/fwanalog.curr* $outdir/fwanalog.new*.log $outdir/convdate.sed \
 		$outdir/fwanalog.analog.conf.ver $outdir/fwanalog.analog.conf.gen $outdir/match_last_line.*
+
+    # Delete the lock file
+    rm -f $outdir/fwanalog.lock
 }
 
 iptables () 
@@ -879,7 +896,7 @@ freebsd ()
 	fi
 
 	#              1             2                         3              4         5          6         7        8         9     
-	$perl -pwe "s!^(\d+/\w+/\d+) ([0-9:]+)\.\d+ *[0-9]*x? +(\w+) @.+ . ([0-9.]+),*(\d*) -\\> ([0-9.]+),*(\d*) PR (\w+) len (\d+).+\$!\$4 - - [\$1:\$2 $timezone] \"GET /$reqhost/\$8/\$7/ HTTP/1.0\" 200 \$9 \"http://\$5/\" \"\" 0 \$3!" \
+	$perl -pwe "s!^(\d+/\w+/\d+) ([0-9:]+)\.\d+ *[0-9]*x? +(\w+) @.+ . ([0-9a-f.:]+),*(\d*) -\\> ([0-9a-f.:]+),*(\d*) PR (\w+) len (\d+).+\$!\$4 - - [\$1:\$2 $timezone] \"GET /$reqhost/\$8/\$7/ HTTP/1.0\" 200 \$9 \"http://\$5/\" \"\" 0 \$3!" \
 		$outdir/fwanalog.current.withmonth > $outdir/fwanalog.current.log
 
 	# $outdir/fwanalog.current.log now contains the data in the Analog URL format.
@@ -936,6 +953,51 @@ ipchains ()
 	# $outdir/fwanalog.current.log now contains the data in the Analog URL format.
 }
 
+
+ipfw ()
+{
+    # fwanalog extension for freebsds ipfw 
+    # 15/Sept/2002 Peter Hunkirchen <phunkirchen@t-online.de> 
+
+    # Parse ipfw logfiles into an analog-compatible "URL log"
+
+    $zegrep -h "Deny" $inputfiles \
+        > $outdir/fwanalog.current
+      
+    mkdateconvscript
+    # Create script to convert lines without year to fully specified date
+
+    $sed -f $outdir/convdate.sed $outdir/fwanalog.current > $outdir/fwanalog.current.withyear
+    # Use the script on the current logfile
+       
+    # Example of converted log line:
+    # 2002 Sep 15 07:47:04 yepp /kernel: ipfw: 65435 Deny UDP 80.133.123.52:1042 165.132.149.211:4665 out via tun0
+    # Example of desired output:
+    # 131....38 - - [31/Mar/2001:00:58:17 +0200] "GET /212....31/TCP/21 HTTP/1.0" 200 \
+    #       44 "61636" "00....:00" 10 eth1
+    #
+    # Which means:
+    # ip - - [date] "GET Desthost/Protocol/Port" 200 PcktLen "http://Sourceport/" "Macadr" 0 interface
+    # Sourceport is in the referrer field, macadr in the user-agent, interface
+    # in the VirtualHost.
+    # There is no MAC address in ipchains logs.
+       
+    # Decide if the source or the destination host is included in the
+    # Blocked Packet Report (option "onehost" in fwanalog.opts)
+    if [ $onehost = true ]; then
+        reqhost="\$8"                           # The analog "request" contains the source ip
+    elif [ $onehost =  dynip ]; then
+        reqhost="firewall"                      # The analog "request" contains this string
+    else
+        reqhost="\$10"                          # The analog "request" contains the destination ip
+    fi
+
+    #               1      2      3     4           5             6        7         8     9         10    11       12	      13
+    $perl -pwe "s!^(\d+) +(\w+) +(\d+) ([0-9:]+) .+(Deny|Reject) ([\w-]+) ([0-9.]+):(\d*) ([0-9.]+):(\d*) ([\w-]+) ([\w-]+) ([\w-]+)\$!\$7 - - [\$3/\$2/\$1:\$4 $timezone] \"GET /$reqhost/\$6/\$10/ HTTP/1.0\" 200 1 \"http://\$8/\" \"\" 0 \$13 !" \$outdir/fwanalog.current.withyear > $outdir/fwanalog.current.log
+
+    # $outdir/fwanalog.current.log now contains the data in the Analog URL format.
+}
+
 zynos () 
 {
 	# Parse ZynOS (ZyXEL, NETGEAR) logfiles into an analog-compatible "URL log"
@@ -1053,8 +1115,8 @@ cisco ()
 					.+ Inbound.TCP.connection.denied \s	# PIX ID (?); verbose description
 					from \s ([0-9.]+)/([0-9]+)	\s		# Source IP ($5), port ($6)
 					to \s ([0-9.]+)/([0-9]+)	\s		# Destination IP ($7), port ($8)
-                    flags \s ([A-Z 	]*) \s+				# TCP flags ($9)
-					on.interface.([a-zA-Z0-9&_-]+)[ \t]*	# interface ($10), possible whitespace
+                    flags \s ([A-Z 	]*) [ \t]+			# TCP flags ($9)
+					(?:on.interface.)?([a-zA-Z0-9&_-]*)[ \t]*	# interface ($10), possible whitespace
                 $							# End of line :-)
               !$5 - - [$3/$2/$1:$4 '$timezone'] \"GET /'$reqhost'/tcp/$8/ HTTP/1.0" 200 0 "http://$6/" "" 0 $10!x' \
 		| ${perl} -pwe \
@@ -1207,7 +1269,7 @@ cisco ()
 					(?:\d{4}\s)?				# optional year when timestamp is switched on
                     ([0-9:]+)				\s+	# syslog time $4
                     .+							# uninteresting data
-					: \s list\s(\d+)\sdenied \s # rule number $5
+					: \s list\s([^ ]+)\sdenied \s # rule number $5
 					(\w+) \s					# protocol $6
 					([0-9.]+)\(?(\d*)\)?\s(->)\s# Source IP $7, optional port $8; $9 just for compatibility with $reqhost_1
 					([0-9.]+)\(?(\d*)\)?		# Dest IP $10, optional port $11
@@ -1289,6 +1351,19 @@ watchguard ()
                 $							# End of line
               !$8 - $12 [$3/$2/$1:$4 '$timezone'] \"GET /'$reqhost'/$7/$10/ HTTP/1.0" 200 $6 "" "" 0 $5!x' \
 		| ${perl} -pwe \
+            's! 				# IGMP
+                ^									# Begin of line
+                    (\d+)\s+(\w+)\s+(\d+)	\s+		# syslog year $1 month $2 day $3
+                    ([0-9:]+)				\s+		# syslog time $4
+					[0-9a-zA-Z_.-]+\s[a-z]+\[\d+\]:\s # ip/hostname, process name, PID
+					deny\s[a-z]+ \s ([a-z]+\d+) \s	# deny in/out, interface $5
+					(\d+)\s(igmp) \s \d+\s\d+\s	# Packet length $6, protocol $7
+					([0-9.]+) \s ([0-9.]+) \s		# Source, dest IP $8, $9
+					([^ ]*) \s* (.*)				# Some optional info $10
+					\(([a-zA-Z0-9_. -]*)\) \s		# Reason $12
+                $							# End of line
+              !$8 - $12 [$3/$2/$1:$4 '$timezone'] \"GET /'$reqhost'/$7/$10/ HTTP/1.0" 200 $6 "" "" 0 $5!x' \
+		| ${perl} -pwe \
             's! 				# TCP and UDP
                 ^									# Begin of line
                     (\d+)\s+(\w+)\s+(\d+)	\s+		# syslog year $1 month $2 day $3
@@ -1376,8 +1451,83 @@ fw1 ()
               !$12 - $15 [$2/$3/$4:$5 '$timezone'] \"GET /'$reqhost'/$14/$11/ HTTP/1.0" 200 0 "http://$16/" "$15" 0 $7!x' \
 		$outdir/fwanalog.current.withyear > $outdir/fwanalog.current.log
 
-        #     cp $outdir/fwanalog.current.log $outdir/fwanalog.save.log	
+	# $outdir/fwanalog.current.log now contains the data in the Analog URL format.
+}
 
+sonicwall ()
+{
+	# Parse SonicWall TZ-170 syslog logfiles into an analog-compatible
+	# "URL log"
+
+	$zegrep -h " connection dropped" $inputfiles \
+		> $outdir/fwanalog.current
+
+	mkdateconvscript
+	# Create script to convert lines without year to fully specified date
+
+	$sed -f $outdir/convdate.sed $outdir/fwanalog.current > $outdir/fwanalog.current.withyear
+	# Use the script on the current logfile
+
+	# Example of converted log line:
+	# 2004 Dec  9 14:50:38 192.168.1.1 id=firewall sn=0123456789AB \
+	# time="2004-12-09 15:07:38" fw=400.300.200.100 pri=5 c=64 m=36 \
+	# msg="TCP connection dropped" n=38533 src=66.167.62.131:2930:WAN \
+	# dst=400.300.200.100:135:WAN
+
+	# Example of desired output:
+	# 66.167.62.131 - - [9/Dec/2004:14:50:38 -0500] \
+	# "GET /400.300.200.100/TCP/135 HTTP/1.0" 200 36 \
+	# "http://2930/" "-" 0 WAN
+	#
+	# Which means:
+	# ip - - [date] \
+	# "GET Desthost/Protocol/Port HTTP/1.0" 200 PcktLen \
+	# "http://Sourceport/" "Mac" 0 interface
+
+	# Sourceport is in the referrer field, macadr in the user-agent,
+	# interface in the VirtualHost.
+
+	# Decide if the source or the destination host is included in the
+	# Blocked Packet Report (option "onehost" in fwanalog.opts)
+	if [ $onehost = true ]; then
+		reqhost="\$7"				# The analog "request" contains the source ip
+	elif [ $onehost = dynip ]; then
+		reqhost="firewall"			# The analog "request" contains this string
+	else
+		reqhost="\$10"				# The analog "request" contains the destination ip
+	fi
+	
+	$perl -pwe 's!^
+	(\d+)          \s+  #    1 year
+	(\w+)          \s+  #    2 month
+	(\d+)          \s+  #    3 day
+	([0-9:]+)      \s+  #    4 time
+	[0-9.]+        \s+  # skip IP-address
+	\w+=\w+        \s+  # skip id=firewall
+	\w+=\w+        \s+  # skip sn=serial-number
+	\w+=\"[0-9-]+  \s+  # skip time="yyyy-mm-dd
+	  [0-9:]+\"    \s+  #        hh:mm:ss"
+	\w+=[0-9.]+    \s+  # skip fw=IP-address
+	\w+=\d+        \s+  # skip pri=num
+	\w+=\d+        \s+  # skip c=num
+	\w+=                # skip m=
+	(\d+)          \s+  #    5 packet-length
+	\w+=\"              # skip msg="
+	(\w+)          \s+  #    6 protocol
+	\w+\s+\w+\"    \s+  # skip connection dropped"
+	\w+=\d+        \s+  # skip n=num
+	\w+=                # skip src=
+	([0-9.]+)\:         #    7 source-IP:
+	(\d+)\:             #    8 source-port:
+	(\w+)          \s+  #    9 source-interface
+	\w+=                # skip dst=
+	([0-9.]+)\:         #   10 dest-IP:
+	(\d+)\:             #   11 dest-port:
+	\w+                 # skip dest-interface
+	.+                  # skip the-rest
+	!$7 - - [$3/$2/$1:$4 '$timezone'] \"GET /'$reqhost'/$6/$11/ HTTP/1.0\" 200 $5 \"http://$8/\" \"-\" 0 $9!x' \
+		$outdir/fwanalog.current.withyear > $outdir/fwanalog.current.log
+ 
 	# $outdir/fwanalog.current.log now contains the data in the Analog URL format.
 }
 
@@ -1466,8 +1616,10 @@ clean_up ()
 	#	No arguments
 	#####
 
-	true
-	# a no-op so the shell doesn't complain
+    # Delete the lock file
+    if [ -e $outdir/fwanalog.lock ]; then
+        rm -f $outdir/fwanalog.lock
+    fi
 }
 
 
